package Back_Pressure;

import org.junit.jupiter.api.Test;
import org.reactivestreams.Subscriber;
import org.reactivestreams.Subscription;
import reactor.core.publisher.Flux;

public class BackPressureEx {

    @Test
    public void subscriberRequest() {
        //the ability for the consumer to signal the producer that the rate of emission is manage

        //subscriber == request(N) to manually handles the requests.
        //publisher == limitRate() Ensure the backpressure signals from downstream subscriber are split into batches, effectively rate limiting the upstream publisher

        //Flux.range(1, 20).log().subscribe();
        Flux.range(1, 20).log().subscribe(new Subscriber<Integer>() {
            Subscription subscription;
            int requestNo = 3;
            int count = 0;

            @Override
            public void onSubscribe(Subscription subscription) {
                this.subscription = subscription;
                this.subscription.request(requestNo);
            }

            @Override
            public void onNext(Integer integer) {
                count++;
                if (count % 3 == 0) {
                    subscription.request(requestNo);
                }
            }

            @Override
            public void onError(Throwable throwable) {
                System.out.println("Error:" + throwable);
            }

            @Override
            public void onComplete() {
                System.out.println("OnComplete: Success");
            }
        });
    }

        @Test
        public void subscriberRequestFourth(){
            Flux.range(1,20).log().subscribe(s-> System.out.println("Data =="+s),
                    err-> System.out.println("Error==="+err),
                    ()-> System.out.println("Success"),
                    subscription -> subscription.request(10));
        }

    @Test
    public void PubliserLimitRate(){
        Flux.range(1,20).log().limitRate(3).subscribe(System.out::println);
        //Flux.range(1,10).retry().log().subscribe();
    }

}
