package Flux_sample_operators;

import org.junit.jupiter.api.Test;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.ArrayList;
import java.util.List;

public class FluxFilterMapAndFlatMap {

    @Test
    public void fluxFilter(){
        Flux<Integer> range = Flux.range(1,100);

        range.filter(n->n%2==0).subscribe(d->System.out.println("Sub1 Received : "+d),
                err->System.out.println("Sub1 error : "+err),
                ()->System.out.println("Sub1 Successful : "));

        range.filter(n->n%2!=0).subscribe(d->System.out.println("Sub2 Received : "+d),
                err->System.out.println("Sub2 error : "+err),
                ()->System.out.println("Sub2 Successful : "));
    }

    @Test
    public void fluxMap(){
        Flux.range(1,10).filter(n->n<5).map(n->n*2).subscribe(System.out::println);

        Flux.range(1,10).map(n->Double.valueOf(n)).log().subscribe();

        Flux.range(1,10).map(n->Flux.just(Double.valueOf(n))).log().subscribe();
    }

    @Test
    public void fluxFlatMap(){
        Flux.range(1,10).flatMap(n->Flux.just(Double.valueOf(n))).log().subscribe();

        Flux.just(1,2,3).flatMap(n->{
            System.out.println("inside flatmap value ========");
            return Flux.range(n,n*2);
        }).log().subscribe();
    }

    public static Mono<List<String>> monoOfList() {
        List<String> list = new ArrayList<>();
        list.add("one");
        list.add("two");
        list.add("three");
        list.add("four");

        return Mono.just(list);
    }

    @Test
    public void monoTofluxUsingFlatMapMany() {
// Mono to flux
        FluxFilterMapAndFlatMap.monoOfList()
                .flatMapMany(Flux::fromIterable)
                .log().subscribe(System.out::println);
//mono to flux
        FluxFilterMapAndFlatMap.monoOfList().flatMapIterable(list -> list).log().subscribe(System.out::println);
    }

}
